@@ -1,12 +1,26 @@
 (* approx: proxy server for Debian archive files
-   Copyright (C) 2009  Eric C. Cooper <ecc@cmu.edu>
+   Copyright (C) 2010  Eric C. Cooper <ecc@cmu.edu>
    Released under the GNU General Public License *)
 
 open Util
 open Config
 open Log
 
-type paragraph = (string * string) list
+type paragraph = { file : string; line : int; fields : (string * string) list }
+
+exception Missing of paragraph * string
+
+let defined name par = List.mem_assoc name par.fields
+
+let lookup name par =
+  try List.assoc name par.fields
+  with Not_found -> raise (Missing (par, name))
+
+let file_name par = par.file
+
+let line_number par = par.line
+
+let iter_fields proc par = List.iter proc par.fields
 
 let trim_left s i =
   let n = String.length s in
@@ -35,22 +49,22 @@ let parse line =
     name, info
   with _ -> failwith ("malformed line: " ^ line)
 
-let read_paragraph chan =
+let read_paragraph file n chan =
   let trim s =
     substring s ~until: (trim_right s (String.length s))
   in
-  let rec loop lines =
+  let rec loop lines i j =
     let next =
       try Some (trim (input_line chan))
       with End_of_file -> None
     in
     match next with
     | None ->
-        if lines <> [] then lines
+        if lines <> [] then lines, i, j + 1
         else raise End_of_file
     | Some "" ->
-        if lines <> [] then lines
-        else loop []
+        if lines <> [] then lines, i, j + 1
+        else loop [] (i + 1) (j + 1)
     | Some line ->
         if line.[0] = ' ' || line.[0] = '\t' then
           match lines with
@@ -59,25 +73,26 @@ let read_paragraph chan =
                 if line = " ." then ""
                 else substring line ~from: 1
               in
-              loop ((last ^ "\n" ^ line) :: others)
+              loop ((last ^ "\n" ^ line) :: others) i (j + 1)
           | [] -> failwith ("leading white space: " ^ line)
         else
-          loop (line :: lines)
+          loop (line :: lines) i (j + 1)
   in
-  List.rev_map parse (loop [])
+  let fields, i, j = loop [] n n in
+  { file = file; line = i; fields = List.rev_map parse fields }, j
 
 let fold f init file =
   let read_file chan =
-    let next () =
-      try Some (read_paragraph chan)
+    let next n =
+      try Some (read_paragraph file n chan)
       with End_of_file -> None
     in
-    let rec loop x =
-      match next () with
-      | Some p -> loop (f x p)
+    let rec loop x n =
+      match next n with
+      | Some (p, n') -> loop (f x p) n'
       | None -> x
     in
-    loop init
+    loop init 1
   in
   with_in_channel open_file file read_file
 
@@ -95,22 +110,35 @@ let read file =
 
 (* Not used yet:
 
+(* A more efficient alternative to map that builds the result list
+   in reverse order *)
+
 let rev_map f = fold (fun acc p -> f p :: acc) []
 
+(* Map a function over each paragraph in a Debian control file *)
+
 let map f file = List.rev (rev_map f file)
 
+(* A more efficient alternative to filter that builds the result list
+   in reverse order *)
+
+val rev_filter : (paragraph -> bool) -> string -> paragraph list
+
 let rev_filter f = fold (fun acc p -> if f p then p :: acc else acc) []
 
+(* Return a list of paragraphs satisfying a predicate *)
+
 let filter f file = List.rev (rev_filter f file)
 
 *)
 
-let get_checksum fields =
-  try List.assoc "sha256" fields, file_sha256sum
-  with Not_found ->
-    try List.assoc "sha1" fields, file_sha1sum
-    with Not_found ->
-      List.assoc "md5sum" fields, file_md5sum
+let get_checksum par =
+  if defined "sha256" par then
+    lookup "sha256" par, file_sha256sum
+  else if defined "sha1" par then
+    lookup "sha1" par, file_sha1sum
+  else
+    lookup "md5sum" par, file_md5sum
 
 type info = string * int64
 
@@ -129,6 +157,8 @@ let read_checksum_info file =
   let lines, checksum = get_checksum (read file) in
   info_list lines, checksum
 
+let lookup_info field par = info_list (lookup field par)
+
 type validity =
   | Valid
   | Wrong_size of int64
