@@ -1,13 +1,37 @@
 (* approx: proxy server for Debian archive files
-   Copyright (C) 2007  Eric C. Cooper <ecc@cmu.edu>
+   Copyright (C) 2010  Eric C. Cooper <ecc@cmu.edu>
    Released under the GNU General Public License *)
 
 (* The format of Debian control files is defined in
    http://www.debian.org/doc/debian-policy/ch-controlfields.html *)
 
-(* A paragraph is represented as a list of (field, value) pairs *)
+(* Abstract type respresenting a paragraph in a control file *)
 
-type paragraph = (string * string) list
+type paragraph
+
+(* Name of control file from which paragraph was read *)
+
+val file_name : paragraph -> string
+
+(* Line number at which paragraph starts *)
+
+val line_number : paragraph -> int
+
+(* Apply a procedure to each (field, value) pair *)
+
+val iter_fields : (string * string -> unit) -> paragraph -> unit
+
+(* Exception raised when a field lookup fails *)
+
+exception Missing of paragraph * string
+
+(* Check if a field is present *)
+
+val defined : string -> paragraph -> bool
+
+(* Find the value corresponding to a field name, or raise Missing *)
+
+val lookup : string -> paragraph -> string
 
 (* Fold a function over each paragraph in a Debian control file *)
 
@@ -22,28 +46,6 @@ val iter : (paragraph -> unit) -> string -> unit
 
 val read : string -> paragraph
 
-(* Not used yet:
-
-(* Map a function over each paragraph in a Debian control file *)
-
-val map : (paragraph -> 'a) -> string -> 'a list
-
-(* A more efficient alternative to map that builds the result list
-   in reverse order *)
-
-val rev_map : (paragraph -> 'a) -> string -> 'a list
-
-(* Return a list of paragraphs satisfying a predicate *)
-
-val filter : (paragraph -> bool) -> string -> paragraph list
-
-(* A more efficient alternative to filter that builds the result list
-   in reverse order *)
-
-val rev_filter : (paragraph -> bool) -> string -> paragraph list
-
-*)
-
 (* Return the strongest checksum information in a paragraph,
    along with the corresponding checksum function *)
 
@@ -57,6 +59,10 @@ type info = string * int64
 
 val info_list : string -> (info * string) list
 
+(* Apply info_list to the value of a field *)
+
+val lookup_info : string -> paragraph -> (info * string) list
+
 (* Read a single-paragraph control file and return a pair consisting of the
    list of ((checksum, size), filename) lines and the checksum function *)
 
