@@ -1,5 +1,5 @@
 (* approx: proxy server for Debian archive files
-   Copyright (C) 2009  Eric C. Cooper <ecc@cmu.edu>
+   Copyright (C) 2010  Eric C. Cooper <ecc@cmu.edu>
    Released under the GNU General Public License *)
 
 open Util
@@ -13,9 +13,8 @@ let message enabled level =
      otherwise syslog-ng behaves differently than syslog *)
   let terminate str =
     let n = String.length str in
-    if n = 0 then "\n"
-    else if str.[n - 1] = '\n' then str
-    else str ^ "\n"
+    if n = 0 || str.[n - 1] <> '\n' then str ^ "\n"
+    else str
   in
   Printf.ksprintf (fun str -> if enabled then syslog log level (terminate str))
 
